/*
	(c) 2021 Dmitry Grinberg   https://dmitry.gr
	Non-commercial use only OR licensing@dmitry.gr
*/

#include "partitions.h"
#include "printf.h"
#include "util.h"


//there seems to be another valid marker (chpt looks at 0x3fb8 to have 0x32957),
// but on boot instead we look at 0x255c to have 0x11954 (seemingly a valid ufs
// marker is used to tell if partition table is valid

#define VALIDATOR_SECTOR		18
#define VALIDATOR_WORD			87
#define VALIDATOR_VALUE			0x0011954

#define PARTTAB_SECTOR			31
#define PARTTAB_MAGIC			0x00032957
#define PARTTAB_VALID			0x00000001

struct UltrixPartition {
	uint32_t len;
	uint32_t start;
};

struct UltrixPartTab {
	uint32_t pad[110];
	uint32_t magic;
	uint32_t valid;
	struct UltrixPartition part[8];
};


bool partMgrFindBootPartition(PartMgrDiskBlockRead readF, void *readD, void *buf512, uint32_t *startSecP, uint32_t *numSecP)
{
	struct UltrixPartTab *pt = (struct UltrixPartTab*)buf512;
	uint32_t *validator = (uint32_t*)buf512;
	
	if (!readF(readD, validator, VALIDATOR_SECTOR))
		fatal("Failed to read validator\n");
	else if (validator[VALIDATOR_WORD] != VALIDATOR_VALUE)
		fatal("Validator value is invalid\n");
	else if (!readF(readD, pt, PARTTAB_SECTOR))
		fatal("Failed to read part tab\n");
	else if (pt->magic != PARTTAB_MAGIC || pt->valid != PARTTAB_VALID)
		fatal("Parttab invalidn\n");
	else {
		
		if (startSecP)
			*startSecP = pt->part[0].start;
		if (numSecP)
			*numSecP = pt->part[0].len;
		
		return true;
	}
	return false;
}